@@ -1,6 +1,6 @@
 # PyAudio : Python Bindings for PortAudio.
 
-# Copyright (c) 2006-2012 Hubert Pham
+# Copyright (c) 2006 Hubert Pham
 
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -106,18 +106,17 @@ Overview
 """
 
 __author__ = "Hubert Pham"
-__version__ = "0.2.8"
+__version__ = "0.2.12"
 __docformat__ = "restructuredtext en"
 
-import sys
+import locale
 
 # attempt to import PortAudio
 try:
     import _portaudio as pa
 except ImportError:
-    print("Please build and install the PortAudio Python " +
-          "bindings first.")
-    sys.exit(-1)
+    print("Could not import the PyAudio C module '_portaudio'.")
+    raise
 
 ############################################################
 # GLOBALS
@@ -196,6 +195,10 @@ paOutputUnderflow = pa.paOutputUnderflow #: Buffer underflow in output
 paOutputOverflow  = pa.paOutputOverflow  #: Buffer overflow in output
 paPrimingOutput   = pa.paPrimingOutput   #: Just priming, not playing yet
 
+###### portaudio misc ######
+
+paFramesPerBufferUnspecified = pa.paFramesPerBufferUnspecified
+
 ############################################################
 # Convenience Functions
 ############################################################
@@ -246,7 +249,7 @@ def get_portaudio_version():
     """
     Returns portaudio version.
 
-    :rtype: string
+    :rtype: int
     """
 
     return pa.get_version()
@@ -296,7 +299,7 @@ class Stream:
                  output=False,
                  input_device_index=None,
                  output_device_index=None,
-                 frames_per_buffer=1024,
+                 frames_per_buffer=pa.paFramesPerBufferUnspecified,
                  start=True,
                  input_host_api_specific_stream_info=None,
                  output_host_api_specific_stream_info=None,
@@ -472,7 +475,7 @@ class Stream:
 
     def get_output_latency(self):
         """
-        Return the input latency.
+        Return the output latency.
 
         :rtype: float
         """
@@ -562,7 +565,7 @@ class Stream:
            Defaults to None, in which this value will be
            automatically computed.
         :param exception_on_underflow:
-           Specifies whether an exception should be thrown
+           Specifies whether an IOError exception should be thrown
            (or silently ignored) on buffer underflow. Defaults
            to False for improved performance, especially on
            slower platforms.
@@ -587,22 +590,26 @@ class Stream:
                         exception_on_underflow)
 
 
-    def read(self, num_frames):
+    def read(self, num_frames, exception_on_overflow=True):
         """
         Read samples from the stream.  Do not call when using
         *non-blocking* mode.
 
         :param num_frames: The number of frames to read.
+        :param exception_on_overflow:
+           Specifies whether an IOError exception should be thrown
+           (or silently ignored) on input buffer overflow. Defaults
+           to True.
         :raises IOError: if stream is not an input stream
           or if the read operation was unsuccessful.
-        :rtype: string
+        :rtype: bytes
         """
 
         if not self._is_input:
             raise IOError("Not input stream",
                           paCanNotReadFromAnOutputOnlyStream)
 
-        return pa.read_stream(self._stream, num_frames)
+        return pa.read_stream(self._stream, num_frames, exception_on_overflow)
 
     def get_read_available(self):
         """
@@ -988,7 +995,8 @@ class PyAudio:
         device_name = device_info.name
 
         # Attempt to decode device_name
-        for codec in ["utf-8", "cp1252"]:
+        os_encoding = locale.getpreferredencoding(do_setlocale=False)
+        for codec in [os_encoding, "utf-8"]:
             try:
                 device_name = device_name.decode(codec)
                 break
